angular.module('myApp', [
    'ionic',
    'myApp.controllers',
    'myApp.services',
    'myApp.directives',
    // 'myApp.filters',
    'ui.bootstrap'
], function($httpProvider) {
    // Use x-www-form-urlencoded Content-Type
    $httpProvider.defaults.headers.post['Content-Type'] = 'application/x-www-form-urlencoded;charset=utf-8';

    /**
     * The workhorse; converts an object to x-www-form-urlencoded serialization.
     * @param {Object} obj
     * @return {String}
     */
    var param = function(obj) {
        var query = '', name, value, fullSubName, subName, subValue, innerObj, i;

        for(name in obj) {
            value = obj[name];

            if(value instanceof Array) {
                for(i = 0; i < value.length; ++i) {
                    subValue = value[i];
                    fullSubName = name + '[' + i + ']';
                    innerObj = {};
                    innerObj[fullSubName] = subValue;
                    query += param(innerObj) + '&';
                }
            }
            else if(value instanceof Object) {
                for(subName in value) {
                    subValue = value[subName];
                    fullSubName = name + '[' + subName + ']';
                    innerObj = {};
                    innerObj[fullSubName] = subValue;
                    query += param(innerObj) + '&';
                }
            }
            else if(value !== undefined && value !== null) {
                query += encodeURIComponent(name) + '=' + encodeURIComponent(value) + '&';
            }
        }

        return query.length ? query.substr(0, query.length - 1) : query;
    };

    // Override $http service's default transformRequest
    $httpProvider.defaults.transformRequest = [function(data) {
        return angular.isObject(data) && String(data) !== '[object File]' ? param(data) : data;
    }];
})

.run(function ($ionicPlatform, $rootScope, currencyService, locationProviderService, dataSavingService, shareDataService, viewport) {

    $rootScope.appVersion = "1.2.3";

    // onDeviceReady is triggered when cordova is loaded
    document.addEventListener("deviceready", onDeviceReady, false);
    function onDeviceReady() {
        // Hide Cordova splash screen
        navigator.splashscreen.hide();

        // Initialize Device details
        $rootScope.deviceUUID         = device.uuid;
        $rootScope.deviceModel        = device.model;
        $rootScope.deviceVersion      = device.version;
        $rootScope.deviceManufacturer = device.manufacturer;

        // add event listeners
        document.addEventListener('pause', onPause.bind(this), false);
        document.addEventListener('resume', onResume.bind(this), false);
        document.addEventListener('online', onOnline, false);
        document.addEventListener('offline', onOffline, false);
    };

    /**
     * The app has been suspended
     */
    function onPause() {
    };

    /**
     * The app has been resumed
     */
    function onResume() {
        locationProviderService.updateLocation();
    };

    /**
     * App loses internet connection
     */
    function onOffline() {
    };

    /**
     * App gains internet connection
     */
    function onOnline() {
        ionic.EventController.trigger('networkReconnected');
        currencyService.updateCurrencyRates();
    };

    $ionicPlatform.ready(function () {
        // Hide the accessory bar by default (remove this to show the accessory bar above the KEYBOARD for form inputs)
        if(window.cordova && window.cordova.plugins.Keyboard) {
            cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
            cordova.plugins.Keyboard.disableScroll(true);
        }
    });

    // Initialize Currencies
    if(dataSavingService.loadItem('actualCurrency') != null) {
        currencyService.setActualCurrency(dataSavingService.loadItem('actualCurrency'));
    } else {
        currencyService.setActualCurrency('EUR');
    }
    currencyService.updateCurrencyRates();

    // Initialize Distance unit
    if(dataSavingService.loadItem('distanceUnit') != null) {
        locationProviderService.setDistanceUnit(dataSavingService.loadItem('distanceUnit'));
    } else {
        locationProviderService.setDistanceUnit('metric');
    }
})

.config(function($stateProvider, $urlRouterProvider, $ionicConfigProvider) {

    // Disable native scrolling
    // $ionicConfigProvider.scrolling.jsScrolling(false);

    // if none of the above states are matched, use this as the fallback
    $urlRouterProvider.otherwise('app/main');

    // setup an abstract state for the menu
    $stateProvider.state('app', {
        url: '/app',
        abstract: true,
        templateUrl: 'templates/menu.html',
        controller: 'MenuController'
    })

    .state('app.currencies', {
        url: '/currencies',
        views: {
            'menuContent': {
                templateUrl: 'templates/currencies.html',
                controller: 'CurrenciesController'
            }
        }
    })

    .state('app.information', {
        url: '/information',
        views: {
            'menuContent': {
                templateUrl: 'templates/information.html',
                controller: 'InformationController'
            }
        }
    })

    .state('app.terms', {
        url: '/terms',
        views: {
            'menuContent': {
                templateUrl: 'templates/terms.html',
                controller: 'TermsController'
            }
        }
    })

    .state('app.privacy', {
        url: '/privacy',
        views: {
            'menuContent': {
                templateUrl: 'templates/privacy.html',
                controller: 'PrivacyController'
            }
        }
    })

    .state('app.about', {
        url: '/about',
        views: {
            'menuContent': {
                templateUrl: 'templates/about.html',
                controller: 'AboutController'
            }
        }
    })

    .state('app.main', {
        url: '/main',
        views: {
            'menuContent': {
                templateUrl: 'templates/main.html',
                controller: 'MainController'
            }
        }
    })

    .state('app.tour-list', {
        url: '/tour-list',
        cache: false,
        views: {
            'menuContent': {
                templateUrl: 'templates/tour-list.html',
                controller: 'TourListController'
            }
        }
    })

    .state('app.tour', {
        url: '/tour',
        cache: true,
        views: {
            'menuContent': {
                templateUrl: 'templates/tour.html',
                controller: 'TourController'
            }
        }
    })

    .state('app.tour-reviews', {
        url: '/tour-reviews',
        cache: true,
        views: {
            'menuContent': {
                templateUrl: 'templates/tour-reviews.html',
                controller: 'TourReviewsController'
            }
        }
    })

    .state('app.tour-contact', {
        url: '/tour-contact',
        cache: true,
        views: {
            'menuContent': {
                templateUrl: 'templates/tour-contact.html',
                controller: 'TourContactController'
            }
        }
    })

    .state('app.tour-contact-confirmation', {
        url: '/tour-contact-confirmation',
        cache: true,
        views: {
            'menuContent': {
                templateUrl: 'templates/tour-contact-confirmation.html',
                controller: 'TourContactConfirmationController'
            }
        }
    })

    .state('app.tour-booking-options', {
        url: '/tour-booking-options',
        cache: true,
        views: {
            'menuContent': {
                templateUrl: 'templates/tour-booking-options.html',
                controller: 'TourBookingOptionsController'
            }
        }
    })

    .state('app.tour-booking-reservation', {
        url: '/tour-booking-reservation',
        cache: true,
        views: {
            'menuContent': {
                templateUrl: 'templates/tour-booking-reservation.html',
                controller: 'TourBookingReservationController'
            }
        }
    })

    .state('app.tour-payment', {
        url: '/tour-payment',
        cache: true,
        views: {
            'menuContent': {
                templateUrl: 'templates/tour-payment.html',
                controller: 'TourPaymentController'
            }
        }
    })

    .state('app.payment-gender', {
        url: '/payment-gender',
        cache: true,
        views: {
            'menuContent': {
                templateUrl: 'templates/payment-gender.html',
                controller: 'PaymentGenderSelectController'
            }
        }
    })

    .state('app.payment-nationality', {
        url: '/payment-nationality',
        cache: true,
        views: {
            'menuContent': {
                templateUrl: 'templates/payment-nationality.html',
                controller: 'PaymentNationalitySelectController'
            }
        }
    })

    .state('app.payment-expiration-month', {
        url: '/payment-expiration-month',
        cache: true,
        views: {
            'menuContent': {
                templateUrl: 'templates/payment-expiration-month.html',
                controller: 'PaymentExpirationMonthController'
            }
        }
    })

    .state('app.payment-expiration-year', {
        url: '/payment-expiration-year',
        cache: true,
        views: {
            'menuContent': {
                templateUrl: 'templates/payment-expiration-year.html',
                controller: 'PaymentExpirationYearController'
            }
        }
    })

    .state('app.tour-booking-confirmation', {
        url: '/tour-booking-confirmation',
        cache: true,
        views: {
            'menuContent': {
                templateUrl: 'templates/tour-booking-confirmation.html',
                controller: 'TourBookingConfirmationController'
            }
        }
    });

});