angular.module('myApp.controllers', [])

.controller('MenuController', function ($scope, $rootScope, currencyService) {
	$scope.currentCurrency      = currencyService.getActualCurrency();
	$scope.actualCurrency       = $scope.currentCurrency['currency'];
	$scope.currencySymbols      = currencyService.getCurrencySymbols();
	$scope.actualCurrencySymbol = $scope.currencySymbols[$scope.actualCurrency];

	ionic.EventController.on('currencyChanged', function() {
		$scope.currentCurrency      = currencyService.getActualCurrency();
		$scope.actualCurrency       = $scope.currentCurrency['currency'];
		$scope.currencySymbols      = currencyService.getCurrencySymbols();
		$scope.actualCurrencySymbol = $scope.currencySymbols[$scope.actualCurrency];
	});
})

/**
 * Currencies page controller
 */
.controller('CurrenciesController', function($scope, $rootScope, $filter, currencyService, locationProviderService, $state, $ionicSideMenuDelegate) {

	var menu = $ionicSideMenuDelegate.$getByHandle('main-menu');
	$scope.toggleMenu = function() {
		menu.toggleLeft();
		ionic.EventController.trigger('currencyChanged');
	}

	// Currencies
	$scope.currencyRates   = currencyService.getCurrencyRates();
	$scope.currencySymbols = currencyService.getCurrencySymbols();
	$scope.currentCurrency = currencyService.getActualCurrency();

	$scope.selectNewCurrency = function(newCurrency) {
		currencyService.setActualCurrency(newCurrency);
		$scope.currentCurrency = currencyService.getActualCurrency();
	};

	// $scope.currentDistanceUnit = locationProviderService.getDistanceUnit();

	// $scope.selectNewDistanceUnit = function(newDistanceUnit) {
	// 	locationProviderService.setDistanceUnit(newDistanceUnit);
	// 	$scope.currentDistanceUnit = locationProviderService.getDistanceUnit();
	// };
})

/**
 * Currencies page controller
 */
.controller('InformationController', function($scope, $rootScope, $state) {

	// App version
	$scope.appVersion = $rootScope.appVersion;

})

/**
 * Main page controller
 */
.controller("MainController", function ($scope, $rootScope, $state, $ionicModal, focus, $ionicSideMenuDelegate, shareDataService, locationProviderService, $ionicPlatform) {

	$ionicSideMenuDelegate.$getByHandle('main-menu').canDragContent(false);

	// Detect first start of the application
	// if (shareDataService.getData('isFirstStart') == null) {
	// 	var isFirstStart = true;
	// 	shareDataService.registerObj('isFirstStart', false);
	// } else {
	// 	var isFirstStart = false;
	// }
	// var isFirstStartFromSave = shareDataService.getData('isFirstStartFromSave');
	// if (isFirstStart == true && isFirstStartFromSave == true) {
	// 	$state.go('app.welcome');
	// }

	// initial search value
	$scope.searchValue = { label: "Rome, Paris, Amsterdam..." };
            console.log(111111)
	// Getting user's location
	$ionicPlatform.ready(function() {
		locationProviderService.updateLocation();
	});
	ionic.EventController.on('locationDefined', function() {
		$scope.userLocation = locationProviderService.getLocation();
	});

	// Open / close the city-search-modal
	$ionicModal.fromTemplateUrl('templates/city-search-modal.html', {
		scope: $scope,
		animation: 'slide-in-up'
	}).then(function (modal) {
		$scope.searchModal = modal;
	});
	$scope.searchCities = function () {
		$scope.searchModal.show();
		//focus('city-search-input');
	};
	$scope.closeSearch = function () {
		$scope.searchModal.hide();
	};
})

.controller("CitySearchModalController", function ($scope, $rootScope, $state, $http, locationProviderService, dataSavingService, notificationService, shareDataService, $ionicLoading) {

	$scope.isResults = false;
	$scope.isCities = false;
	$scope.cities = null;


	$scope.popularDestinations = [
	{
		category: "Cities",
		cityID: "1776",
		cityName: "Budapest",
		label: "Budapest, Hungary"
	},
	{
		category: "Cities",
		cityID: "2058",
		cityName: "Dublin",
		label: "Dublin, Ireland"
	},
	{
		category: "Cities",
		cityID: "3836",
		cityName: "Barcelona",
		label: "Barcelona, Spain"
	},
	{
		category: "Cities",
		cityID: "1093",
		cityName: "Prague",
		label: "Prague, Czech Republic"
	},
	{
		category: "Cities",
		cityID: "1498",
		cityName: "Berlin",
		label: "Berlin, Germany"
	}];

	/**
	 * Loads the results from https://secure.hostelculture.com/com.freetour.mobile.api/iOS/1.2.0/get_cities.php based on the keys the user types
	 * @param  {String} searchTerm
	 */
	$scope.fetchResults = function (searchTerm) {
		if (searchTerm.length >= 2) {
			$scope.isResults = true;

			var url = 'https://secure.hostelculture.com/com.freetour.mobile.api/iOS/1.2.0/get_cities.php?search-term=' + searchTerm;
			$http({
				method: 'GET',
				url: url,
				cache: false
			}).then(function (response) {
				if (response.data.error == null) {

					if (response.data.cities != null) {
						$scope.cities = response.data.cities;
						$scope.isResults = true;
						$scope.isCities = true;
					} else {
						$scope.cities = null;
						$scope.isResults = false;
						$scope.isCities = false;
					}

				} else {
					$ionicLoading.hide();
					notificationService.showMessage(response.data.errorMessage, 2500);
				}
			}, function () {
				$scope.isResults = false;
				$scope.isCities = false;
				$scope.cities = null;

				notificationService.showMessage('Connection failed', 2500);
			});
		} else {
			$scope.isResults = false;
			$scope.isCities = false;
			$scope.cities = null;
		}
	};

	/**
	 * passes the selected city to the next page
	 * @param  {Object} item
	 */
	$scope.findTours = function (item) {
		shareDataService.registerObj('cityObject', item);

		if ($state.current.name == "app.tour-list") {
			$state.go($state.current, {}, { reload: true });
		} else {
			$state.go('app.tour-list');
		}
		$scope.searchModal.hide();
	};

	/**
	 * Calculates the user's current location and broadcasts an event when it is defined
	 */
	$scope.findToursAtCurrentLocation = function () {
		$ionicLoading.show({
			template: 'Loading...'
		});

		var location = locationProviderService.getLocation();
		var url = "https://secure.hostelculture.com/com.freetour.mobile.api/iOS/1.2.0/get_city_id.php?city-name=" + location.cityName + "&country-name=" + location.countryName;

		$http({
			method: 'GET',
			url: url,
			cache: false
		}).then(function (response) {
			if(response.data.error != 'databaseError') {
				$ionicLoading.hide();
				$scope.findTours(response.data);
			} else {
				$ionicLoading.hide();
				notificationService.showMessage(response.data.errorMessage, 2500);
			}
		}, function () {
			$ionicLoading.hide();
			notificationService.showMessage('Connection failed', 2500);
		});
	};
})

/**
 * Tour list page controller
 */
.controller("TourListController", function ($scope, $http, $cacheFactory, shareDataService, locationProviderService, $filter, currencyService, $ionicPopup, popUpService, $state, $ionicLoading, notificationService, $ionicModal) {

	// Getting user's location
	$scope.userLocation = locationProviderService.getLocation();

	// Search Modal
	$ionicModal.fromTemplateUrl('templates/city-search-modal.html', {
		scope: $scope,
		animation: 'slide-in-up'
	}).then(function (modal) {
		$scope.searchModal = modal;
	});
	// Open/Close the searchModal
	$scope.searchCities = function () {
		$scope.searchModal.show();
		//focus('city-search-input');
	};
	$scope.closeSearch = function () {
		$scope.searchModal.hide();
	};

	// Filter Modal
	$ionicModal.fromTemplateUrl('templates/tour-list-filter-modal.html', {
		scope: $scope,
		animation: 'slide-in-up'
	}).then(function (modal) {
		$scope.filterModal = modal;
	});
	// Open/Close the filterModal
	$scope.showFilters = function() {
		$scope.filterModal.show();
	};
	$scope.closeFilters = function() {
		$scope.filterModal.hide();
	};

	// Receiving the city from shareDataService
	$scope.cityObject = shareDataService.getData('cityObject');

	// Gets the selected currency
	$scope.actualCurrency = currencyService.getActualCurrency();

	// Format title to proper length
	if($scope.cityObject.label.length > 18) {
		if($scope.cityObject.cityName.length > 18) {
			$scope.title = $scope.cityObject.label.substring(0, 15) + '...';
		} else {
			$scope.title = $scope.cityObject.cityName;
		}
	} else {
		$scope.title = $scope.cityObject.label;
	}

	// Tours data
	$scope.featuredToursToDisplay = [];
	$scope.toursToDisplay         = [];
	$scope.featuredTours          = [];
	$scope.tours                  = [];
	$scope.resultsNum             = 0;
	$scope.isResults              = true;
	$scope.moreToursCanBeLoaded   = true;

	// Filters data
	$scope.appliedFilters     = 0;
	$scope.languageFilterMode = 'Any';
	$scope.categoryFilterMode = 'Any';
	$scope.budgetFilterMode   = 'Any';

	ionic.EventController.on('networkReconnected', function() {
		$scope.loadTours();
	});

	/**
	 * Sets initial properties for tours
	 * @param  {array} tours
	 * @return void
	 */
	function initializeTours(tours) {
		tours.forEach(function(element, index, array) {
			// Tour badge
			switch(element.tour_rec) {
				case "0":
					element.badge       = null;
					element.badgeBG     = null;
					element.recommended = false;
					break;
				case "1":
					element.badge       = null;
					element.badgeBG     = null;
					element.recommended = true;
					break;
				case "2":
					element.badge       = "Best Seller";
					element.badgeBG     = "balanced-bg";
					element.recommended = false;
					break;
				case "3":
					element.badge       = "Best Seller";
					element.badgeBG     = "balanced-bg";
					element.recommended = true;
					break;
				case "4":
					element.badge       = "Very Popular";
					element.badgeBG     = "energized-bg";
					element.recommended = false;
					break;
				case "5":
					element.badge       = "Very Popular";
					element.badgeBG     = "energized-bg";
					element.recommended = true;
					break;
				default:
					element.badge       = null;
					element.badgeBG     = null;
					element.recommended = false;
			}

			// Tour profile picture
			if(element.tour_profile_picture != null) {
				element.profilePictureStyle = {"background-image":"url('https://admin.freetour.com/images/tours/" + element.tour_id + "/" + element.tour_profile_picture.fn + "')"};
			} else {
				element.profilePictureStyle = {"background-image":"url('https://www.freetour.com/images/default%20pic-01.png'"};
			}

			// Tour price
			element.convertedPrice = currencyService.convert(element.tour_price);
			if(element.convertedPrice > 999) {
				element.fractions = 0;
			} else {
				element.fractions = 2;
			}
		});
	}

	/**
	 * Loads tours from database
	 */
	$scope.loadTours = function() {
		$ionicLoading.show({
			template: 'Loading...'
		});

		var url = 'https://secure.hostelculture.com/com.freetour.mobile.api/iOS/1.2.0/get_tours.php?city-id=' + $scope.cityObject.cityID + '&hash=' + Date.now();

		$http({
			method: 'GET',
			url: url,
			cache: false
		}).then(function(response) {
			$scope.featuredTours = response.data.featuredTours;
			$scope.tours         = response.data.tours;
			$scope.languages     = response.data.info.languages;
			$scope.categories    = response.data.info.categories;
			$scope.resultsNum    = response.data.info.resultsNum;

			// Initializing variables for ALL tours through filtering
			initializeTours($scope.featuredTours);
			initializeTours($scope.tours);

			$scope.displayedTours         = 10;
			$scope.toursToDisplay         = $scope.tours.slice($scope.displayedTours - 10, $scope.displayedTours);
			$scope.featuredToursToDisplay = $scope.featuredTours;
			$scope.displayedTours         = $scope.displayedTours + 10;

			if($scope.resultsNum > 0) {
				$scope.isResults = true;
			} else {
				$scope.isResults = false;
			}

			$ionicLoading.hide();

		}, function () {
			$ionicLoading.hide();
			notificationService.showMessage('Connection failed', 2500);
		});
	}
	$scope.loadTours();

	/**
	 * Loads the next 10 blocks of tours to the view
	 * @return void
	 */
	$scope.updateTours = function() {
		var filteredTours = filterTours($scope.tours, $scope.budgetFilterMode, $scope.languageFilterMode, $scope.categoryFilterMode);
		$scope.toursToDisplay.push.apply($scope.toursToDisplay, filteredTours.slice($scope.displayedTours - 10, $scope.displayedTours));

		if($scope.displayedTours >= $scope.resultsNum) {
			$scope.moreToursCanBeLoaded = false;
		} else {
			$scope.moreToursCanBeLoaded = true;
		}
		$scope.displayedTours = $scope.displayedTours + 10;

		$scope.$broadcast('scroll.infiniteScrollComplete');
	}

	$scope.changeLanguageFilterMode = function(language) {
		$scope.languageFilterMode = language;
	};
	$scope.changeCategoryFilterMode = function(category) {
		$scope.categoryFilterMode = category;
	};
	$scope.changeBudgetFilterMode = function(budget) {
		$scope.budgetFilterMode = budget;
	};

	/**
	 * Filters array of tours based on budget, language and category
	 * @param  {array} tours
	 * @param  {string} budget
	 * @param  {string} language
	 * @param  {string} category
	 * @return {array}
	 */
	function filterTours(tours, budget, language, category) {
		var filteredTours = [];
		filteredTours = $filter('filter')(tours, function (element) {
			if((element.tour_languages.indexOf(language) != -1 || language == 'Any') && (element.tour_type == category || category == 'Any') && (element.budget_filter == budget || budget == 'Any')) {
				return true;
			} else {
				return false;
			}
		});

		return filteredTours;
	}
	/**
	 * Updates the list of tours in the view
	 * @param  {array} tours
	 * @param  {string} budget
	 * @param  {string} language
	 * @param  {string} category
	 * @return void
	 */
	$scope.filterResults = function(budget, language, category) {
		$scope.appliedFilters         = 0;
		$scope.displayedTours         = 10;
		var filteredFeaturedTours     = filterTours($scope.featuredTours, budget, language, category);
		var filteredTours             = filterTours($scope.tours, budget, language, category);
		$scope.featuredToursToDisplay = filteredFeaturedTours.slice(0, 10);
		$scope.toursToDisplay         = filteredTours.slice(0, 10);

		$scope.resultsNum     = filteredFeaturedTours.length + filteredTours.length;
		$scope.displayedTours = $scope.displayedTours + 10;
		if($scope.displayedTours >= $scope.resultsNum) {
			$scope.moreToursCanBeLoaded = false;
		} else {
			$scope.moreToursCanBeLoaded = true;
		}

		if($scope.languageFilterMode != 'Any') {
			$scope.appliedFilters += 1;
		}
		if($scope.categoryFilterMode != 'Any') {
			$scope.appliedFilters += 1;
		}
		if($scope.budgetFilterMode != 'Any') {
			$scope.appliedFilters += 1;
		}

		$scope.filterModal.hide();
	};

	/**
	 * Loads the requested tour
	 * @param  {int} tourID
	 */
	$scope.getTour = function(tourID) {
		$ionicLoading.show({
			template: 'Loading...'
		});

		var url = 'https://secure.hostelculture.com/com.freetour.mobile.api/android/1.2.0/get_tour.php?tour-id=' + tourID + '&hash=' + Date.now();

		$http({
			method: 'GET',
			url: url,
			cache: false
		}).then(function (response) {
			if(response.data.error != 'databaseError') {
				$ionicLoading.hide();

				var tourData = response.data;
				shareDataService.registerObj('tourData', tourData);

				$state.go('app.tour');
			} else {
				$ionicLoading.hide();
				notificationService.showMessage(response.data.errorMessage, 2500);
			}

		}, function () {
			$ionicLoading.hide();
			notificationService.showMessage('Connection Lost', 2500);
		});
	};
})

/**
 * Tour controller
 */
.controller('TourController', function ($scope, shareDataService, currencyService, $sce, viewport, $state, $ionicHistory, $ionicModal) {
	$scope.tour     = shareDataService.getData('tourData');
	$scope.viewport = viewport();

	$ionicModal.fromTemplateUrl('templates/tour-gallery-modal.html', {
		scope: $scope,
		animation: 'slide-in-up'
	}).then(function (modal) {
		$scope.galleryModal = modal;
	});

	// Reviews
	$scope.range = function(n) {
        return new Array(n);
    };
    $scope.showReviews = function() {
    	$state.go('app.tour-reviews');
    };

	// Prices
	$scope.actualCurrency = currencyService.getActualCurrency();
	$scope.tourPrice      = currencyService.convert($scope.tour.tour_price);
	if($scope.tourPrice > 999) {
		$scope.fracionSize = 0;
	} else {
		$scope.fracionSize = 2;
	}

	// Pictures
	$scope.tourGallery = {
		currentPictureIndex: 0
	};
	$scope.picturesNum = $scope.tour.images.length;
	$scope.showPictures = function() {
		$scope.galleryModal.show();
	};

	// Gallery modal
	$scope.goBack = function() {
		$scope.galleryModal.hide();
	};

	// Languages
	$scope.languageSum = Object.keys($scope.tour.tour_languages).length;

	// Text
	$scope.deliberatelyTrustDangerousSnippet = function(text) {
		return $sce.trustAsHtml(text);
	};

	// Continue Booking
	$scope.continueBooking = function() {
		if($scope.tour.tour_active != 0) {
			$state.go('app.tour-booking-options');
		} else {
			$state.go('app.tour-contact');
		}
	};

	// Map
	$scope.getTourDirections = function() {
		window.location.href = "maps://maps.apple.com/?q=" + $scope.tour.tour_coordinates;
	};
})

/**
 * Tour Reviews controller
 */
.controller('TourReviewsController', function($scope, shareDataService, $state) {
	// Tour data
	$scope.tour = shareDataService.getData('tourData');
	$scope.range = function(n) {
        return new Array(n);
    };
})

/**
 * Tour Contact controller
 */
.controller('TourContactController', function($scope, shareDataService, $state, notificationService, $ionicPopup, $http) {
	// Tour data
	$scope.tour = shareDataService.getData('tourData');

	$scope.contactDetails = {
		name: null,
		email: null,
		message: null
	};

	$scope.sendContactMessage = function() {
		$scope.contactForm.$setSubmitted();
		var isValid = true;

		if($scope.contactForm.$invalid) {
			isValid = false;
		}

		if(isValid) {
			var contactProcessingPopUp = $ionicPopup.show({
				template: '',
				title: 'Sending Message',
				subTitle: '...'
			});

			var url = 'https://secure.hostelculture.com/com.freetour.mobile.api/iOS/1.2.0/tour_contact.php';

			$http({
				method: 'POST',
				url: url,
				data: {
					name: $scope.contactDetails.name,
					email: $scope.contactDetails.email,
					message: $scope.contactDetails.message,
					tourObject: $scope.tour
				},
				cache: false
			}).then(function(response) {
				if(response.data.error == null) {
					contactProcessingPopUp.close();
					$state.go('app.tour-contact-confirmation');
				} else {
					contactProcessingPopUp.close();
					notificationService.showMessage(response.data.errorMessage, 1500);
				}

			}, function() {
				contactProcessingPopUp.close();
				notificationService.showMessage("Connection Lost", 1500);
			});
		} else {
			notificationService.showMessage("Please fill in all required details", 1500);
		}
	};
})

/**
 * Tour Contact Confirmation controller
 */
.controller('TourContactConfirmationController', function($scope, shareDataService, $state, $ionicHistory) {
	// Tour data
	$scope.tour = shareDataService.getData('tourData');

	shareDataService.removeAll();
	// clear out the history and cache
	$ionicHistory.clearHistory();
	$ionicHistory.clearCache();
})

/**
 * Tour Booking Options controller - Language & Time
 */
.controller('TourBookingOptionsController', function($scope, shareDataService, currencyService, $state) {
	// Tour data
	$scope.tour = shareDataService.getData('tourData');
	$scope.languageSum = Object.keys($scope.tour.tour_languages).length;

	// Current currency symbol
	$scope.actualCurrency = currencyService.getActualCurrency();

	// Select booking option
	$scope.selectOption = function(time, languageObject, language) {
		$state.go('app.tour-booking-reservation');
		shareDataService.registerObj('tourTime', time);
		shareDataService.registerObj('tourLanguageObject', languageObject);
		shareDataService.registerObj('tourLanguage', language);
	};
})

/**
 * Tour Booking Reservation controller - Date & People
 */
.controller('TourBookingReservationController', function($scope, $rootScope, $http, shareDataService, currencyService, $state, $ionicScrollDelegate, $ionicModal, $ionicScrollDelegate) {
	// Tour data
	$scope.tour               = shareDataService.getData('tourData');
	$scope.tourLanguage       = shareDataService.getData('tourLanguage');
	$scope.tourLanguageObject = shareDataService.getData('tourLanguageObject');

	// Tour time
	$scope.tourTime     = shareDataService.getData('tourTime');
	var tourTimeValue   = $scope.tourTime.split(" ");
	var tourTimeHour    = parseInt(tourTimeValue[0].split(":")[0]);
	var tourTimeMinutes = parseInt(tourTimeValue[0].split(":")[1]);
	if(tourTimeValue[1] != null && tourTimeValue[1].toUpperCase() == "PM") {
		var tourTimeHour = tourTimeHour + 12;
	}

	if($scope.tour.country_tz != null) {
		var providerTime = moment().tz($scope.tour.country_tz);
	} else {
		var providerTime = moment();
	}
	var providerTimeHour      = providerTime.hours();
	var providerTimeMinutes   = providerTime.minutes();
	var isNotBookableToday    = false;
	var isNotBookableTomorrow = false;

	if(tourTimeHour < providerTimeHour || (tourTimeHour == providerTimeHour && tourTimeMinutes <= providerTimeMinutes)) {
		isNotBookableToday = true;

		if($scope.tour.tour_adv == 24 || $scope.tour.tour_adv == 48) {
			isNotBookableTomorrow = true;
		}
	} else {
		if($scope.tour.tour_adv == 24) {
			isNotBookableToday = true;
		}

		if($scope.tour.tour_adv == 48) {
			isNotBookableTomorrow = true;
		}
	}

	// Modals **********************************************
	$ionicModal.fromTemplateUrl('templates/tour-date-modal.html', {
		scope: $scope,
		animation: 'slide-in-up'
	}).then(function (modal) {
		$scope.tourDateModal = modal;
	});
	$scope.closeTourDates = function() {
		$scope.tourDateModal.hide();
	};

	$ionicModal.fromTemplateUrl('templates/tour-booking-type-info-modal.html', {
		scope: $scope,
		animation: 'slide-in-up'
	}).then(function (modal) {
		$scope.tourBookingTypeInfoModal = modal;
	});
	$scope.closeBookingTypeInfo = function() {
		$scope.tourBookingTypeInfoModal.hide();
	};

	// People **********************************************
	if($scope.tour.tourPricing != null && $scope.tour.tourPricing != "" && $scope.tour.tourPricing.type == "ranges") {
		$scope.people = parseInt($scope.tour.tourPricing.pricing[0][0]);
	} else {
		$scope.people = parseInt($scope.tour.tourMin);
 	}
	$scope.addPeople = function() {
		if($scope.people < $scope.tour.tourLimit) {
			if($scope.tour.tourPricing != null && $scope.tour.tourPricing != "" && $scope.tour.tourPricing.type == "ranges") {
				$rangeLength = $scope.tour.tourPricing.pricing.length;
				if($scope.people < $scope.tour.tourPricing.pricing[$rangeLength - 1][1]) {
					$scope.people = $scope.people + 1;
				}
			} else {
				$scope.people = $scope.people + 1;
 			}
		}

		setPrices();
	};
	$scope.subtractPeople = function() {
		if($scope.people > $scope.tour.tourMin) {
			$scope.people = $scope.people - 1;
		}

		setPrices();
	};

	// Booking type **********************************************
	$scope.guaranteedBookingType = true;
	$scope.confirmedBookingType  = false;
	$scope.showBookingTypeInfoModal = function(bookingType) {
		$scope.bookingType = bookingType;
		$scope.tourBookingTypeInfoModal.show();
	};
	$scope.changeBookingType = function(bookingType) {
		if(bookingType == 'guaranteed') {
			$scope.guaranteedBookingType = true;
			$scope.confirmedBookingType  = false;
		} else {
			$scope.guaranteedBookingType = false;
			$scope.confirmedBookingType  = true;
		}

		$scope.tourBookingTypeInfoModal.hide();
		setPrices();
	};

	// Current currency symbol **************************************
	$scope.actualCurrency = currencyService.getActualCurrency();

	// Prices
	$scope.isGroupPriced = false;
	if($scope.tour.tour_price == 0) {
		$scope.tourPrice     = 0;
		$scope.isTourPayable = false;
		$scope.initiallyFree = true;
	} else {
		$scope.tourPrice     = $scope.tour.tour_price;
		$scope.isTourPayable = true;
		$scope.initiallyFree = false;
	}

	setPrices();
	function setPrices() {
		if($scope.tour.tourPricing != null && $scope.tour.tourPricing != "") {
			// Price ranges
			if($scope.tour.tourPricing.type == "ranges") {
				// Find the right range
				$i = 0;
				$rangeLength = $scope.tour.tourPricing.pricing.length;
				while($i + 1 <= $rangeLength && $scope.tour.tourPricing.pricing[$i][0] <= $scope.people) {
					$scope.tourPrice = $scope.tour.tourPricing.pricing[$i][2];
					$i++;
				}
				$scope.totalPrice = $scope.people * $scope.tourPrice;
			// Group pricing
			} else if($scope.tour.tourPricing.type == "group") {
				if($scope.tour.tourPricing.pricing[1] == true) {
					// Constant price for the group
					$scope.isGroupPriced = true;
					$scope.totalPrice    = $scope.tour.tourPricing.pricing[0];
				} else if($scope.tour.tourPricing.pricing[1] == false) {
					// If group price reached, stay on it
					$scope.totalPrice = $scope.people * $scope.tourPrice;
					if($scope.totalPrice > $scope.tour.tourPricing.pricing[0]) {
						$scope.totalPrice    = $scope.tour.tourPricing.pricing[0];
						$scope.isGroupPriced = true;
					} else {
						$scope.isGroupPriced = false;
					}
				}
			}
		// Regular pricing
		} else {
			if($scope.initiallyFree == true) {
				if($scope.people > $scope.tour.tourUpTo) {
					$scope.tourPrice        = $scope.tour.tourFee;
					$scope.tourFeeConverted = currencyService.convert($scope.tour.tourFee);
					$scope.isTourPayable    = true;
				} else {
					$scope.tourPrice = 0;
					$scope.isTourPayable = false;
				}
			}
			$scope.totalPrice = $scope.people * $scope.tourPrice;
		}

		// Booking type (credit card details are required: more than 4 people, recent booking, required by company)
		if(($scope.people >= 4 || $scope.deviceRecentlyBooked || $scope.tour.ccreq == 1) && $scope.isTourPayable == false) {
			$scope.guaranteedBookingType = true;
			$scope.confirmedBookingType  = false;
		}

		// Converting prices & calculating deposit price
		$scope.tourPriceConverted    = currencyService.convert($scope.tourPrice);
		$scope.totalPriceConverted   = currencyService.convert($scope.totalPrice);
		$scope.depositPrice          = $scope.totalPrice * $scope.tour.tourComission;
		$scope.depositPriceConverted = currencyService.convert($scope.depositPrice);

		// Booking fee (Standard Flexible booking: 1 EUR)
		if($scope.guaranteedBookingType && $scope.isTourPayable) {
			$scope.bookingFee = $scope.people;
			$scope.bookingFeeConverted = currencyService.convert($scope.bookingFee);
		} else {
			$scope.bookingFee = 0;
			$scope.bookingFeeConverted = 0;
		}

		$ionicScrollDelegate.resize();
	}

	// Dates **********************************************
	// Setting all days initially to false
	$scope.tourAvailableDays = {
		'Mon': false,
		'Tue': false,
		'Wed': false,
		'Thu': false,
		'Fri': false,
		'Sat': false,
		'Sun': false
	};
	var weekDay = new Array(7);
	weekDay[0] = "Sun";
	weekDay[1] = "Mon";
	weekDay[2] = "Tue";
	weekDay[3] = "Wed";
	weekDay[4] = "Thu";
	weekDay[5] = "Fri";
	weekDay[6] = "Sat";

	// Setting up the available days
	$scope.tourLanguageObject.days.forEach(function(element, index, array) {
		$scope.tourAvailableDays[element.substring(0, 3)] = true;
	});

	// Calculating the first available day of the tour
	$scope.data          = {};
	$scope.data.tourDate = new Date();
	var dayOfWeek        = weekDay[$scope.data.tourDate.getDay()];
	var isFirstDayFound  = false;
	var i                = 0;
	while(!isFirstDayFound) {
		var isDayClosed = false;
		$scope.tour.closedDates.forEach(function(element, index, array) {
			var closedDate = new Date(element);
			if($scope.data.tourDate.getDate() === closedDate.getDate() && $scope.data.tourDate.getMonth() === closedDate.getMonth() && $scope.data.tourDate.getFullYear() === closedDate.getFullYear()) {
				isDayClosed = true;
			}
		});

		if(i == 0 && isNotBookableToday) {
			isDayClosed = true;
		}
		if(i == 1 && isNotBookableTomorrow) {
			isDayClosed = true;
		}
		i++;

		if($scope.tourAvailableDays[dayOfWeek] == true && isDayClosed == false) {
			isFirstDayFound = true;
		} else {
			$scope.data.tourDate.setDate($scope.data.tourDate.getDate() + 1);
			dayOfWeek = weekDay[$scope.data.tourDate.getDay()];
		}
	}
	$scope.tourDateConverted    = moment($scope.data.tourDate).format("YYYY MMMM DD");
	$scope.tourDateConvertedDay = moment($scope.data.tourDate).format("dddd");

	// Change dates
	$scope.changeTourDates = function() {
		$scope.tourDateModal.show();
	};

	// Closed days
	var today = new Date();
	var tomorrow = new Date();
	tomorrow.setDate(today.getDate() + 1);
	$scope.disabled = function(date, mode) {
		if(mode === 'day') {
			if(isNotBookableToday && date.getDate() === today.getDate() && date.getMonth() === today.getMonth() && date.getFullYear() === today.getFullYear()) {
				return true;
			}
			if(isNotBookableTomorrow && date.getDate() === tomorrow.getDate() && date.getMonth() === tomorrow.getMonth() && date.getFullYear() === tomorrow.getFullYear()) {
				return true;
			}

			if($scope.tourAvailableDays[weekDay[date.getDay()]] == true) {
				var isDayClosed = false;
				$scope.tour.closedDates.forEach(function(element, index, array) {
					var closedDate = new Date(element);
					if(date.getDate() === closedDate.getDate() && date.getMonth() === closedDate.getMonth() && date.getFullYear() === closedDate.getFullYear()) {
						isDayClosed = true;
					}
				});

				return isDayClosed;
			} else {
				return true;
			}
		} else {
			return false;
		}
	};

	$scope.minDate = new Date();

	$scope.changeDate = function() {
		shareDataService.registerObj('tourDate', $scope.data.tourDate);
		$scope.data.tourDate = shareDataService.getData('tourDate');
		$scope.tourDateConverted = moment($scope.data.tourDate).format("YYYY MMMM DD");
		$scope.tourDateConvertedDay = moment($scope.data.tourDate).format("dddd");
		$scope.tourDateModal.hide();
	};

	$scope.continueBooking = function() {
		shareDataService.registerObj('tourDate', $scope.data.tourDate);
		shareDataService.registerObj('totalPrice', $scope.totalPrice);
		shareDataService.registerObj('bookingFee', $scope.bookingFee);
		shareDataService.registerObj('depositPrice', $scope.depositPrice);
		shareDataService.registerObj('tourPeopleNum', $scope.people);
		shareDataService.registerObj('guaranteedBookingType', $scope.guaranteedBookingType);
		shareDataService.registerObj('confirmedBookingType', $scope.confirmedBookingType);
		$state.go('app.tour-payment');
	};
})

/**
 * Tour Payment page controller
 */
.controller('TourPaymentController', function($scope, $rootScope, shareDataService, currencyService, $http, $location, $anchorScroll, $state, notificationService, popUpService, $ionicPopup, $ionicHistory) {

	// Current currency symbol
	$scope.actualCurrency = currencyService.getActualCurrency();

	// receieving data from shareDataService - Tour information
	$scope.tour                  = shareDataService.getData('tourData');
	$scope.tourPeopleNum         = shareDataService.getData('tourPeopleNum');
	$scope.tourTime              = shareDataService.getData('tourTime');
	$scope.tourLanguage          = shareDataService.getData('tourLanguage');
	$scope.tourDate              = shareDataService.getData('tourDate');
	$scope.tourDate              = moment($scope.tourDate).format("YYYY-MM-DD");
	$scope.guaranteedBookingType = shareDataService.getData('guaranteedBookingType');
	$scope.confirmedBookingType  = shareDataService.getData('confirmedBookingType');

	$scope.totalPrice       = shareDataService.getData('totalPrice');
	$scope.bookingFee       = shareDataService.getData('bookingFee');
	$scope.depositPrice     = shareDataService.getData('depositPrice');
	$scope.payable          = $scope.depositPrice + $scope.bookingFee;
	$scope.payableConverted = currencyService.convert($scope.payable);
	if($scope.totalPrice == 0) {
		$scope.isTourPayable = false;
	} else {
		$scope.isTourPayable = true;
	}

	$scope.paymentDetails = {
		// User details
		firstName: null,
		lastName: null,
		email: null,
		gender: null,
		nationality: null,
		phone: null,
		// Payment details
		cardholderName: null,
		cardNumber: null,
		creditCardType: 'unknown',
		expirationYear: null,
		expirationMonth: null,
		CCVNumber: null,
		terms: false
	};

	// Custom validation variables and functions
	$scope.isGenderValid          = true;
	$scope.isNationalityValid     = true;
	$scope.isArrivalTimeValid     = true;
	$scope.isExpirationMonthValid = true;
	$scope.isExpirationYearValid  = true;
	$scope.isTermsAgreed          = true;

	$scope.getCreditCardType = function(cardNumber) {
		if(/^5[1-5][0-9]{14}$/.test(cardNumber)) {
			$scope.paymentDetails.creditCardType = "MC";
		} else if (/^4[0-9]{12}([0-9]{3})?$/.test(cardNumber)) {
			$scope.paymentDetails.creditCardType = "VISA";
		} else {
			$scope.paymentDetails.creditCardType = 'unknown';
		}
	};

	// Custom validation variables and functions
	$scope.isGenderValid          = true;
	$scope.isNationalityValid     = true;
	$scope.isTermsAgreed          = true;

	$scope.isExpirationMonthValid = true;
	$scope.isExpirationYearValid  = true;

	$scope.termsChanged = function() {
		$scope.isTermsAgreed = $scope.paymentDetails.terms;
	};
	$scope.goToTerms = function() {
		$state.go('app.terms');
	};
	$scope.getGender = function() {
		$state.go('app.payment-gender');
	};
	$scope.getNationality = function() {
		$state.go('app.payment-nationality');
	};
	$scope.getExpirationMonth = function() {
		$state.go('app.payment-expiration-month');
	};
	$scope.getExpirationYear = function() {
		$state.go('app.payment-expiration-year');
	};

	ionic.EventController.on('genderChanged', function() {
		$scope.paymentDetails.gender = shareDataService.getData('gender');
		$scope.isGenderValid = true;
	});

	ionic.EventController.on('nationalityChanged', function() {
		$scope.paymentDetails.nationality = shareDataService.getData('nationality');
		$scope.isNationalityValid = true;
	});

	ionic.EventController.on('expirationMonthChanged', function() {
		$scope.paymentDetails.expirationMonth = shareDataService.getData('expirationMonth');
		$scope.isExpirationMonthValid = true;
	});

	ionic.EventController.on('expirationYearChanged', function() {
		$scope.paymentDetails.expirationYear       = shareDataService.getData('expirationYear');
		$scope.paymentDetails.expirationYear2Digit = moment($scope.paymentDetails.expirationYear, 'YYYY').format("YY");
		$scope.isExpirationYearValid = true;
	});

	$scope.validateBooking = function() {
		$scope.paymentForm.$setSubmitted();
		var isBookable = true;

		if($scope.creditCardType == 'unknown') {
			$scope.paymentForm.cardNumber.$invalid = true;
			if($scope.isTourPayable) {
				isBookable = false;
			}
		}

		if($scope.paymentForm.$invalid) {
			isBookable = false;
		}

		if($scope.paymentDetails.gender == null || $scope.paymentDetails.gender == '') {
			$scope.isGenderValid = false;
			isBookable = false;
		}

		if($scope.paymentDetails.nationality == null || $scope.paymentDetails.nationality.country_name == '') {
			$scope.isNationalityValid = false;
			isBookable = false;
		}

		if($scope.paymentDetails.terms == false) {
			$scope.isTermsAgreed = false;
			isBookable = false;
		}

		if($scope.paymentDetails.expirationMonth == null || $scope.paymentDetails.expirationMonth == '') {
			if($scope.isTourPayable || $scope.guaranteedBookingType) {
				$scope.isExpirationMonthValid = false;
				isBookable = false;
			}
		}

		if($scope.paymentDetails.expirationYear == null || $scope.paymentDetails.expirationYear == '') {
			if($scope.isTourPayable || $scope.guaranteedBookingType) {
				$scope.isExpirationYearValid = false;
				isBookable = false;
			}
		}

		if(isBookable) {
			var paymentProcessingPopUp = $ionicPopup.show({
				template: '',
				title: 'Processing Booking',
				subTitle: '...'
			});
			var url = 'https://secure.hostelculture.com/com.freetour.mobile.api/iOS/1.2.0/tour_payment.php';
			var tourObject = {
				tour_id: $scope.tour.tour_id,
				tourComission: $scope.tour.tourComission,
				tour_owner_id: $scope.tour.tour_owner_id,
				tour_city_id: $scope.tour.tour_city_id,
				tour_name: $scope.tour.tour_name,
				tour_meeting: $scope.tour.tour_meeting,
				tour_coordinates: $scope.tour.tour_coordinates,
				tour_notes: $scope.tour.tour_notes
			}

			$http({
				method: 'POST',
				url: url,
				data: {
					firstName: $scope.paymentDetails.firstName,
					lastName: $scope.paymentDetails.lastName,
					email: $scope.paymentDetails.email,
					phone: $scope.paymentDetails.phone,
					gender: $scope.paymentDetails.gender,
					nationality: $scope.paymentDetails.nationality,
					terms: $scope.paymentDetails.terms,

					isTourPayable: $scope.isTourPayable,
					totalPrice: $scope.totalPrice,
					depositPrice: $scope.depositPrice,
					bookingFee: $scope.bookingFee,
					guaranteedBookingType: $scope.guaranteedBookingType,
					confirmedBookingType: $scope.confirmedBookingType,
					tourObject: tourObject,
					tourPeopleNum: $scope.tourPeopleNum,
					tourTime: $scope.tourTime,
					tourLanguage: $scope.tourLanguage,
					tourDate: $scope.tourDate,

					cardHolderName: $scope.paymentDetails.cardholderName,
					creditCardType: $scope.paymentDetails.creditCardType,
					cardNumber: $scope.paymentDetails.cardNumber,
					expirationMonth: $scope.paymentDetails.expirationMonth,
					expirationYear: $scope.paymentDetails.expirationYear2Digit,
					CCVNumber: $scope.paymentDetails.CCVNumber,

					deviceUUID: $rootScope.deviceUUID,
					appVersion: $rootScope.appVersion
				},
				cache: false
			}).then(function(response) {
				if(response.data.error == null) {
					shareDataService.registerObj('paymentDetails', response.data);
					paymentProcessingPopUp.close();
					$state.go('app.tour-booking-confirmation');

				} else {
					paymentProcessingPopUp.close();
					popUpService.showMessage(response.data.errorTitle, response.data.errorMessage, 2500);
				}

			}, function() {
				paymentProcessingPopUp.close();
				notificationService.showMessage("Connection Lost", 2500);
			});
		} else {
			notificationService.showMessage("Please fill in all required details", 2500);
		}
	};
})

/**
 * Payment Gender page controller
 */
.controller('PaymentGenderSelectController', function($scope, $rootScope, shareDataService, $ionicHistory) {
	$scope.selectGender = function(gender) {
		shareDataService.registerObj('gender', gender);
		ionic.EventController.trigger('genderChanged');
		$ionicHistory.goBack();
	};
})

/**
 * Payment Nationality Select page controller
 */
.controller('PaymentNationalitySelectController', function($scope, $rootScope, shareDataService, $ionicHistory, $http, notificationService) {
	var url = 'https://secure.hostelculture.com/com.freetour.mobile.api/iOS/1.2.0/get_nationalities.php';

	$http({
		method: 'GET',
		url: url,
		cache: false
	}).then(function (response) {
		if (response.data.error == null) {
			$scope.nationalities = response.data;
		} else {
			notificationService.showMessage('Connection failed', 2500);
		}
	}, function () {
		notificationService.showMessage('Connection failed', 2500);
	});

	$scope.selectNationality = function(nationality) {
		shareDataService.registerObj('nationality', nationality);
		ionic.EventController.trigger('nationalityChanged');
		$ionicHistory.goBack();
	};
})

/**
 * Payment Credit Card Expiration Month page controller
 */
.controller('PaymentExpirationMonthController', function($scope, $rootScope, shareDataService, $ionicHistory) {
	$scope.monthOptions = {
		January: '01',
		February: '02',
		March: '03',
		April: '04',
		May: '05',
		June: '06',
		July: '07',
		August: '08',
		September: '09',
		October: '10',
		November: '11',
		December: '12'
	};

	$scope.selectExpirationMonth = function(expirationMonth) {
		shareDataService.registerObj('expirationMonth', expirationMonth);
		ionic.EventController.trigger('expirationMonthChanged');
		$ionicHistory.goBack();
	};
})

/**
 * Payment Credit Card Expiration Year page controller
 */
.controller('PaymentExpirationYearController', function($scope, $rootScope, shareDataService, $ionicHistory) {
	$scope.currentYear = parseInt(moment().format("YYYY"));
	$scope.yearOptions = [];

	for(var i = 0; i < 20; i++) {
		$scope.yearOptions[i] = $scope.currentYear + i;
	}

	$scope.selectExpirationYear = function(expirationYear) {
		shareDataService.registerObj('expirationYear', expirationYear);
		ionic.EventController.trigger('expirationYearChanged');
		$ionicHistory.goBack();
	};
})

/**
 * Tour Booking Confirmation page controller
 */
.controller('TourBookingConfirmationController', function($scope, shareDataService, currencyService, $sce, popUpService, $ionicHistory) {
	$scope.$on('$ionicView.enter', function(e) {
		// Current currency symbol
		$scope.actualCurrency = currencyService.getActualCurrency();

		// Booking Data
		$scope.tour              = shareDataService.getData('tourData');
		$scope.tourPeopleNum     = shareDataService.getData('tourPeopleNum');
		$scope.tourTime          = shareDataService.getData('tourTime');
		$scope.tourLanguage      = shareDataService.getData('tourLanguage');
		$scope.tourDate          = shareDataService.getData('tourDate');
		$scope.tourDateFormatted = moment($scope.tourDate).format("YYYY MMMM DD");
		$scope.tourDateDay       = moment($scope.tourDate).format("dddd");
		$scope.paymentDetails    = shareDataService.getData('paymentDetails');
		$scope.bookingNumber     = $scope.paymentDetails.bookingNumber;
		$scope.isTourPayable     = false;

		// clear out the history and cache
		shareDataService.removeAll();
		$ionicHistory.clearHistory();
		$ionicHistory.clearCache();

		// Texts
		$scope.deliberatelyTrustDangerousSnippet = function(text) {
		  return $sce.trustAsHtml(text);
		};

		// Directions
		$scope.showOnMap = function() {
			window.open("google.navigation:q=" + $scope.tour.tour_coordinates + "&mode=d" , '_system');
		};

		// calculating prices
		$scope.totalPrice     = $scope.paymentDetails.totalPrice;
		$scope.depositPrice   = $scope.paymentDetails.depositPrice;
		$scope.remainderPrice = $scope.totalPrice - $scope.depositPrice;
		$scope.totalPriceConverted     = currencyService.convert($scope.totalPrice);
		$scope.depositPriceConverted   = currencyService.convert($scope.depositPrice);
		$scope.remainderPriceConverted = $scope.totalPriceConverted - $scope.depositPriceConverted;

		if($scope.totalPrice != 0) {
			$scope.isTourPayable = true;
		}

		// Payment email is succeeded?
		if($scope.paymentDetails.emailSuccess == false) {
		  popUpService.showMessage("There has been a problem with sending out your booking confirmation email", "Please contact our costumer care department at support@hostelculture.com or call +353 1 442 84 08", 7000);
		}
	});
})

/**
 * About page controller
 */
.controller('AboutController', function($scope, $http, $sce, $ionicLoading) {
	// $ionicLoading.show({
	// 	template: 'Loading...'
	// });

	var url = 'https://secure.hostelculture.com/com.freetour.mobile.api/iOS/1.2.0/get_content.php?content-id=about';
	$http({
		method: 'GET',
		url: url,
		cache: false
	}).then(function (response) {
		if (response.data.error != 'databaseError') {
			// $ionicLoading.hide();
			$scope.aboutContent = response.data.content;
		} else {
			// $ionicLoading.hide();
			$scope.aboutContent = null;
		}

	}, function () {
		$ionicLoading.hide();
		$scope.aboutContent = null;
	});

	// Text
	$scope.deliberatelyTrustDangerousSnippet = function(text) {
		return $sce.trustAsHtml(text);
	};
})

/**
 * Terms & Conditions page controller
 */
.controller('TermsController', function($scope, $http, $sce, $ionicLoading) {
	// $ionicLoading.show({
	// 	template: 'Loading...'
	// });

	var url = 'https://secure.hostelculture.com/com.freetour.mobile.api/iOS/1.2.0/get_content.php?content-id=terms';
	$http({
		method: 'GET',
		url: url,
		cache: false
	}).then(function (response) {
		if (response.data.error != 'databaseError') {
			// $ionicLoading.hide();
			$scope.termsContent = response.data.content;
		} else {
			// $ionicLoading.hide();
			// $scope.aboutContent = null;
		}

	}, function () {
		// $ionicLoading.hide();
		$scope.termsContent = null;
	});

	// Text
	$scope.deliberatelyTrustDangerousSnippet = function(text) {
		return $sce.trustAsHtml(text);
	};
})

/**
 * Security & Privacy page controller
 */
.controller('PrivacyController', function($scope, $http, $sce, $ionicLoading) {
	// $ionicLoading.show({
	// 	template: 'Loading...'
	// });

	var url = 'https://secure.hostelculture.com/com.freetour.mobile.api/iOS/1.2.0/get_content.php?content-id=privacy';
	$http({
		method: 'GET',
		url: url,
		cache: false
	}).then(function (response) {
		if (response.data.error != 'databaseError') {
			// $ionicLoading.hide();
			$scope.privacyContent = response.data.content;
		} else {
			// $ionicLoading.hide();
			// $scope.aboutContent = null;
		}

	}, function () {
		// $ionicLoading.hide();
		$scope.privacyContent = null;
	});

	// Text
	$scope.deliberatelyTrustDangerousSnippet = function(text) {
		return $sce.trustAsHtml(text);
	};
});
